<?php

namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;


class Df00400ImpostoController extends \Teknisa\DF\Controllers\TeknisaCrud {
    protected $crudService;
    protected $dataSourceName = 'imposto_completo';
    protected $entityManager;
    protected $environment;
    protected $Df00400ImpostoService;

    public function __construct($crudService, $parameterBag, $environment,$entityManager,$Df00400ImpostoService) {
        parent::__construct($crudService, $parameterBag);
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->Df00400ImpostoService = $Df00400ImpostoService;
    }
    public function verificaRange(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'dadosfiscncm_regrafiscal';
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName('dadosfiscncm_regrafiscal');
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array('P_CDIMPOSTO'));
        $this->crudService->find($response, $newFilterCriteria);
        //var_dump($response);die;
    }

    public function findImposto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'imposto_completo';
        parent::find($request,$response);
    }
    public function findContabilizacao(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'cctbimpforn_contctbl';
        parent::find($request,$response);
    }

    public function findBaseCalc(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'basecalc_impostos_encargo';
        parent::find($request,$response);
    }

    public function findOutrosImpostosTipoOperacao(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'basecalctpoper_impostos';
        parent::find($request,$response);
    }

    public function deleteImposto(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'imposto_completo';
        parent::delete($request,$response);
    }

    public function deleteBaseCalc(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'basecalc_impostos_encargo';
        parent::delete($request,$response);
    }

    public function deleteContabilizacao(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'cctbimpforn_contctbl';
        parent::delete($request,$response);
    }

    public function deleteOutrosImpostosTipoOperacao(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'basecalctpoper_impostos';
        parent::delete($request,$response);
    }

    public function salvarOutrosImpostosTipoOperacao(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'basecalctpoper_impostos';
        $row = $request->getRow();
        $conditions = $this->condicaoBaseCalcTipoOperacao($row);
        $resul = $this->realizaConsultaBanco($conditions);
        if(empty($resul->getRows())){
            parent::save($request,$response);
            $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
        }
        else{
            $response->setError(new DTO\Response\Error("Código já cadastrado para CFOP, CST e Código Fiscal informados.", 1));
        }

    }

    public function salvarBaseCalc(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'basecalc';
        $row = $request->getRow();

        if($row['__is_new']){
            if($row['TIPOWIDGET']=='ENCARGO'){
                $conditions = $this->condicaoBaseCalc($row);
            }
            else{
                $conditions = $this->condicaoBaseCalcPadrao($row);
            }
            $resul = $this->realizaConsultaBanco($conditions);
            if(empty($resul->getRows())){
                parent::save($request,$response);
                $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
            }
            else{
                if($row['TIPOWIDGET']=='ENCARGO'){
                    $response->setError(new DTO\Response\Error("Impossível adicionar. Registro já existe.", 1));
                }
                else{
                    $response->setError(new DTO\Response\Error("Código já cadastrado para CFOP, CST e Código Fiscal informados.", 1));
                }
            }
        }else{
            if($row['TIPOWIDGET']=='ENCARGO'){
                $conditions = $this->condicaoBaseCalc($row);
                $resul = $this->realizaConsultaBanco($conditions);
                
                if(empty($resul->getRows())){
                    parent::save($request,$response);
                }
                else{
                    if(($row['CDIMPOSTO'] == $resul->getRows()[0]['CDIMPOSTO'])     && ($row['CDENCARGO'] == $resul->getRows()[0]['CDENCARGO']) &&
                       ($row['NRSEQUBCALC'] == $resul->getRows()[0]['NRSEQUBCALC']) && ($row['IDTRATIMPOBCENC'] != $resul->getRows()[0]['IDTRATIMPOBCENC'])) { //Libera edição em Base de Cálculo -> Encargo -> Tratamento.
                           parent::save($request,$response);
                    }
                    else $response->setError(new DTO\Response\Error("Impossível adicionar. Registro já existe.", 1));
                }
            }else{
                $conditions = $this->condicaoBaseCalcPadrao($row);
                $resul = $this->realizaConsultaBanco($conditions);
                if(empty($resul->getRows())){
                    parent::save($request,$response);
                    $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
                }
                else{
                    $response->setError(new DTO\Response\Error("Código já cadastrado para CFOP, CST e Código Fiscal informados.", 1));
                }
            }
        }
    }

    public function salvarContabilizacao(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'cctbimpforn';
        $row = $request->getRow();
        $conditions = $this->condicaoContabilizacao($row);
        $resul = $this->realizaConsultaBanco($conditions);
        if(empty($resul->getRows())){
            $row->set('__is_new', true);
        }else{
            $row->set('__is_new', false);
        }
        parent::save($request,$response);
    }

    public function salvarImposto(DTO\Request $request, DTO\Response $response) {
        $validacaoSubstTributaria = true;
        $validacaoUtilizarValorImpostos = true;
        try {
            $situacao = $request->getRow()['__is_new'];
            $row = $request->getRow();
            //$edit = $situacao ? $edit='Adicionando' : $edit='Editando';
            //$response->setError(new DTO\Response\Error($edit."-  Imposto: ".$row['CDIMPOSTO'], 1));
            if($situacao){
                if($row['NMIMPOSTRIB']){
                    if($this->verificaImpostoSubstTributaria($row)){
                        $validacaoSubstTributaria = false;
                        $response->setError(new DTO\Response\Error("Operação Bloqueada! Não é possível definir um imposto como um imposto para substituição tributária se o mesmo utilizar o próprio valor do imposto na sua base de cálculo.", 1));
                    }else{
                        $validacaoSubstTributaria = true;
                    }
                }
                if($validacaoSubstTributaria){
                    $this->dataSourceName = 'imposto';
                    parent::save($request,$response);
                    $response->addDataset(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));

                }
            }else{
                if($row['NMIMPOSTRIB']){
                    if($this->verificaImpostoSubstTributaria($row)){
                        $validacaoSubstTributaria = false;
                        $response->setError(new DTO\Response\Error("Operação Bloqueada! Não é possível definir um imposto como um imposto para substituição tributária se o mesmo utilizar o próprio valor do imposto na sua base de cálculo.", 1));
                    }else{
                        $validacaoSubstTributaria = true;
                    }
                }
                if($row['IDUTILPVRIMPBC']=='S'){
                    if($this->verificaUtilizacaoValorImposto($row)){
                        $validacaoUtilizarValorImpostos = false;

                        $response->setError(new DTO\Response\Error("Operação Bloquada! Não é possível utilizar o próprio valor do imposto na sua base de cálculo em um imposto para substituição tributária.", 1));
                    }else{
                        $validacaoUtilizarValorImpostos = true;

                    }
                }
                if($validacaoSubstTributaria && $validacaoUtilizarValorImpostos){
                    if( ( ($row['vlrAntigo_IDORGARRECAD']!== $row['IDORGARRECAD'])||($row['vlrAntigo_IDIMPORECUP'] !== $row['IDIMPORECUP']) || ($row['vlrAntigo_IDFORMAPUR'] !== $row['IDFORMAPUR']) || ($row['vlrAntigo_IDCONSVRIMPAG'] !== $row['IDCONSVRIMPAG']) ) && (!empty($row['CDIMPOSTONFE']) ) ){
                        //$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                        $conditions = $this->condicaoImpostoNfe($row);
                        $this->dataSourceName = 'impostonfe';
                        $resul = $this->realizaConsultaBanco($conditions);
                        $impostonfe = $resul->getRows()[0];
                        $array = [
                            "CDIMPOSTO"=>$impostonfe['CDIMPOSTONFE'],
                            "SGIMPOSTONFE"=>$impostonfe['SGIMPOSTONFE']
                            ];
    
                        $response->addMessage(new Message($array));
                        try {
                            $connection = $this->entityManager->getConnection();
                            $connection->beginTransaction();
                            $this->dataSourceName = 'imposto';
                            parent::save($request,$response);
                            $this->atulizarImpostosRelacionados($row,$response,$request);
                            $this->atulizarImpostoNfe($row,$response,$request);
                            if($row['IDUTALIQTPOPER']=='N'){
                                $this->deletarAliquota($row,$response,$request);
                            }
                            $connection->commit();
                        } catch (\Exception $e) {
                            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
                            $this->entityManager->rollback();
                        }
                    }
                    else{
                        try {
                            $this->dataSourceName = 'imposto';
                            parent::save($request,$response);
                            if($row['IDUTALIQTPOPER']=='N'){
                                $this->deletarAliquota($row,$response,$request);
                            }
                        }catch (\Exception $e) {
                            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
                            $this->entityManager->rollback();
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
        }
    }

    public function atulizarImpostosRelacionados($resul,$response,$request){
        try {
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $this->dataSourceName = "imposto";
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            $array = array();
            $conditions = $this->condicaoImpostoNfe($resul);
            $rows = $this->realizaConsultaBanco($conditions);
            $rows = $rows->getRows();
            for($i = 0; $i<count($rows);$i++){
                $row = new Row();
                $row->set('__is_new',  false);
                $row->set('__createdLocal',true);
                $row->set('changed',true);
                //$row->set('DTULTATUIMPO',$currentTime);
                $row->set('CDIMPOSTO',$rows[$i]['CDIMPOSTO']);
                $row->set('IDORGARRECAD', $resul['IDORGARRECAD']);
                $row->set('IDIMPORECUP', $resul['IDIMPORECUP']);
                $row->set('IDFORMAPUR', $resul['IDFORMAPUR']);
                $row->set('IDCONSVRIMPAG', $resul['IDCONSVRIMPAG']);
                $row->set('NRORG',$this->environment->getNrOrgTrab());
                $array[] = $row;
            }
            $dataSet = new DataSet('imposto', $array);
            $response->addDataSet($dataSet);
            $requestSalvar = new DTO\Request\dataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            $saveRows = $this->crudService->getRowsToSave($requestSalvar,$dataSourceName);
            $this->crudService->save('imposto', $saveRows);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            //var_dump($e);die;
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            //$this->entityManager->rollback();
        }
    }

    public function atulizarImpostoNfe($row,$response,$request){
        try {
            $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $this->dataSourceName = "impostonfe";
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            $array = array();

            $rowSave = new Row();
                $rowSave->set('__is_new',  false);
                $rowSave->set('__createdLocal',true);
                $rowSave->set('changed',true);
                //$rowSave->set('DTULTATUIMPO',$currentTime);
                $rowSave->set('CDIMPOSTONFE',$row['CDIMPOSTONFE']);
                $rowSave->set('IDORGARRECADNFE', $row['IDORGARRECAD']);
                $rowSave->set('IDIMPORECUPNFE', $row['IDIMPORECUP']);
                $rowSave->set('IDFORMAPURNFE', $row['IDFORMAPUR']);
                //$rowSave->set('IDCONSVRIMPAG', $row['IDCONSVRIMPAG']);
                $rowSave->set('NRORG',$this->environment->getNrOrgTrab());
            $array[] = $rowSave;
            $dataSet = new DataSet('impostonfe', $array);
            $requestSalvar = new DTO\Request\dataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            $saveRows = $this->crudService->getRowsToSave($requestSalvar,$dataSourceName);
            $this->crudService->save('impostonfe', $saveRows);
            $this->entityManager->flush();
        }catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            //$this->entityManager->rollback();
        }
    }

    public function deletarAliquota($rows,$response, $request ){
        try {
            $this->dataSourceName = 'aliquota';
            $conditions = $this->condicaoImposto($rows);
            $resul = $this->realizaConsultaBanco($conditions);
            $aliquota = $resul->getRows();
            if(!empty($aliquota)){
                $array = array();
                for($i = 0; $i<count($aliquota);$i++){
                    $row = new Row();
                    $row->set('CDIMPOSTO', $aliquota[$i]['CDIMPOSTO']);
                    $row->set('NRALIQUOTA', $aliquota[$i]['NRALIQUOTA']);
                    $row->set('NRORG', $this->environment->getNrOrgTrab());
                    $row->set('__is_new', false);
                    $array[] = $row;
                }
                $dataSet = new DataSet('aliquota', $array);
                $requestSalvar = new DTO\Request\dataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                $request = $this->crudService->buildDataSetIfRow($requestSalvar, $this->dataSourceName);
                $dataSet = $request->getDataSet();
                $this->Df00400ImpostoService->deletar($dataSet);
                $this->entityManager->flush();
            }
        }catch (\Exception $e) {
            $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
        }
    }

    public function verificaImpostoSubstTributaria($row){
        $this->dataSourceName = 'imposto';
        $conditions = $this->condicaoImpostoSubstTributaria($row);
        $resul = $this->realizaConsultaBanco($conditions);
        $imposto = $resul->getRows()[0];
        if($imposto['IDUTILPVRIMPBC']=='S'){
            return true;
        }else{
            return false;
        }
    }

    public function verificaUtilizacaoValorImposto($row){
        $this->dataSourceName = 'imposto';
        $conditions = $this->condicaoUtilizacaoImposto($row);
        $resul = $this->realizaConsultaBanco($conditions);
        return(! empty($resul->getRows()) );
    }

    public function condicaoImpostoApuracao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          )
        );
        return $conditions;
    }


    public function condicaoImpostoSubstTributaria($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTRIB'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          )
        );
        return $conditions;
    }

    public function condicaoBaseCalc($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          ),
          array(
            "value" => $row['CDENCARGO'],
            "operator" => "=",
            "columnName" => "CDENCARGO"
          )
        );
        return $conditions;
    }

    public function condicaoBaseCalcPadrao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          ),
          array(
            "value" => $row['CDIMPOBCALC'],
            "operator" => "=",
            "columnName" => "CDIMPOBCALC"
          ),
          array(
            "value" => $row['CDCSTIMPO'],
            "operator" => "=",
            "columnName" => "CDCSTIMPO"
          ),
          array(
            "value" => $row['CDFISCIMPO'],
            "operator" => "=",
            "columnName" => "CDFISCIMPO"
          ),
          array(
            "value" => $row['CDCFOPIMPO'],
            "operator" => "=",
            "columnName" => "CDCFOPIMPO"
          ),
          array(
            "value" => $row['IDTRATIMPOBC'],
            "operator" => "=",
            "columnName" => "IDTRATIMPOBC"
          )
        );
        return $conditions;
    }

    public function condicaoBaseCalcTipoOperacao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          ),
          array(
            "value" => $row['CDIMPOBCALC'],
            "operator" => "=",
            "columnName" => "CDIMPOBCALC"
          ),
           array(
            "value" => $row['IDENTRSAIDOP'],
            "operator" => "=",
            "columnName" => "IDENTRSAIDOP"
          ),
          array(
            "value" => $row['CDTIPOOPER'],
            "operator" => "=",
            "columnName" => "CDTIPOOPER"
          ),
          array(
            "value" => $row['CDCSTIMPOTPOP'],
            "operator" => "=",
            "columnName" => "CDCSTIMPOTPOP"
          ),
          array(
            "value" => $row['CDFISCIMPOTPOP'],
            "operator" => "=",
            "columnName" => "CDFISCIMPOTPOP"
          ),
          array(
            "value" => $row['CDCFOPIMPOTPOP'],
            "operator" => "=",
            "columnName" => "CDCFOPIMPOTPOP"
          ),
          array(
            "value" => $row['IDTRATIMPOBCTPOP'],
            "operator" => "=",
            "columnName" => "IDTRATIMPOBCTPOP"
          )
        );
        return $conditions;
    }

    public function condicaoUtilizacaoImposto($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTRIB"
          )
        );
        return $conditions;
    }

    public function condicaoImpostoNfe($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTONFE'],
            "operator" => "=",
            "columnName" => "CDIMPOSTONFE"
          )
        );
        return $conditions;
    }

    public function condicaoImposto($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          ),
           array(
            "operator" => "IS_NOT_NULL",
            "columnName" => "CDTIPOOPER"
          )
        );
        return $conditions;
    }

    public function condicaoContabilizacao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDIMPOSTO'],
            "operator" => "=",
            "columnName" => "CDIMPOSTO"
          ),
           array(
            "value" => $row['IDTPIJURIMPO'],
            "operator" => "=",
            "columnName" => "IDTPIJURIMPO"
          )
        );
        return $conditions;
    }

    public function realizaConsultaBanco($conditions) {
        $filterCriteria = new FilterCriteria($this->dataSourceName,$conditions);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->Df00400ImpostoService->buscar($newFilterCriteria);
        return $dataSet;
    }


}